
/*

  Create a representation of a tape file (including variable block counts)
  in an ordinary file.

  Representation is: each block of data to be preceded by an int, denoting
  the block size.

  Representation can be copied back to tape using `file2tape', to create
  an identical copy of the original tape file.

*/

#include <stdio.h>
#include <fcntl.h>

#define MAX_RECORD_SIZE 80000

#define DELAY 200000     /* microseconds */

static char *usage        = "USAGE: tape2file <device> <file>\n" ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

void get_params (device, file, argc, argv)

char **device, **file ;
int argc ;
char *argv [] ;

{
  if (argc != 3)
  {
    printf (usage) ;
    exit (1) ;
  }
  *device = argv [1] ;
  *file   = argv [2] ;
}

void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char err_mess [128] ;

  char *device, *file ;
  int src_desc, dst_desc ;
  int going = 1 ;

  get_params (&device, &file, argc, argv) ;

  src_desc = open (device, O_RDONLY) ;
  strcpy (err_mess, "Failed to open \"") ;
  strcat (err_mess, device) ;
  strcat (err_mess, "\"") ;
  sys_assert (src_desc >= 0, err_mess) ;

  dst_desc = open (  file, O_CREAT | O_TRUNC | O_WRONLY, 0x1a4) ;
  strcpy (err_mess, "Failed to open \"") ;
  strcat (err_mess, file) ;
  strcat (err_mess, "\"") ;
  sys_assert (dst_desc >= 0, err_mess) ;

  while (going)
  {
    char record [MAX_RECORD_SIZE] ;

    int len ;

    len = read (src_desc, record, MAX_RECORD_SIZE) ;
    sys_assert (len >= 0, "Read failed") ;
    usleep (DELAY) ;  /* Deschedule...otherwise this program hogs virtually
			 *all* the cpu cycles of the machine, probably due
			 to some scheduling cockup involving the exabyte
			 drivers */
    if (len == 0)
    {
      fprintf (stdout, "EOF encountered\n") ;
      going = 0 ;
    }
    else
    {
      int result ;

      result = write (dst_desc, (char *) &len, sizeof (int)) ;
      sys_assert (result >= 0, "Write failed") ;
      result = write (dst_desc, record, len) ;
      sys_assert (result >= 0, "Write failed") ;
    }
  }
  close (dst_desc) ;
  close (src_desc) ;
  exit  (0) ;
}
